﻿using System;
using System.Collections.Generic;
using CQG.Operators;

namespace SimpleMA
{
    /// <summary>
    /// Simple Moving Average CQG operator. This implementation doesn't use state (calculates all period each time).
    /// Also this implementation doesn't ask to fill invalids - this means that any input value could be
    /// double.NaN and this situation should be processed correctly.
    /// </summary>
    [CQGOperator("SimpleMaOp(@, period)", FillInvalids = false)]
    public class SimpleMaOp : IOperator
    {
        /// <summary>
        /// Moving average period
        /// </summary>
        private int _period = 21;

        #region IOperator implementation

        /// <summary>
        /// Implements <see cref="IOperator.Initialize"/>
        /// </summary>
        public int Initialize(IDictionary<string, string> i_params, IBarsRebuilder i_barsRebuilder)
        {
            string value = i_params["period"];
            if (!String.IsNullOrEmpty(value))
            {
                _period = Convert.ToInt32(value);
            }

            return _period;
        }

        /// <summary>
        /// Implements <see cref="IOperator.Calculate"/>
        /// </summary>
        public void Calculate(IList<double>[] i_inputs, double[] io_outputs, ref IDeepCloneable io_state, IList<object> io_primitives)
        {
            double sum = 0;
            foreach (double elem in i_inputs[0])
            {
                if (double.IsNaN(elem))
                {
                    io_outputs[0] = double.NaN;
                    return;
                }
                sum += elem;
            }

            io_outputs[0] = sum / _period;
        }

        #endregion
    }
}