// MyOperatorBase.h is part of CQG COM Operators SDK Sample

#pragma once
#include <vector>
#include <map>

#include "resource.h"       // main symbols
#include "comstr.h"

#import "cqgop.tlb" raw_interfaces_only, no_namespace

// CMyOperatorBase
template< typename State >
class CMyOperatorBase: public ICQGOp
{
public:
   CMyOperatorBase()
      : m_states( 1, State() )
      { m_currentState = m_states.begin(); }

public:
   ///@name part of ICQGOp interface methods
   ///@{
   STDMETHOD(Calculate)(int i_sizeIn, double * i_input, int i_sizeOut, double * o_output)
      { return S_OK; }

   STDMETHOD(InitializeStep)(int i_sizeIn, double * i_lastValidInput)
      { return S_OK; }

   STDMETHOD(GetInputExpressionIndex)(short i_ii, short * o_inputIndex, LPWSTR * o_error)
    {
        if (o_inputIndex)
        {
            *o_inputIndex = i_ii;
            return S_OK;
        }
        return E_INVALIDARG;
    }
    
    STDMETHOD(GetInputBarType)(short i_ii, LPWSTR * o_barTypeText, LPWSTR * o_error)
    {
        return CreateReturnString(o_barTypeText, L"");
    }
    
    STDMETHOD(GetOutputBarType)(short i_ii, LPWSTR * o_barTypeText, LPWSTR * o_error)
    {
        return CreateReturnString( o_barTypeText, L"curve1");
    }

    STDMETHOD(GetOutputScaleType)(short i_ii, short * o_scaleType, short * o_index1, short * o_index2, LPWSTR * o_error)
    {
        if (o_scaleType && o_index1 && o_index2)
        {
            *o_scaleType = OSD_TRANSPARENT;
            *o_index1 = 0;
            *o_index2 = 0;
            return S_OK;
        }
        return E_INVALIDARG;
    }

    STDMETHOD(GetUpdateDelayAndCount)(int i_sizeIn, double * i_inputValues, int * o_outputDelay, int * o_outputBarCount)
    {
        if (o_outputDelay && o_outputBarCount)
        {
            *o_outputDelay = 0;
            *o_outputBarCount = 1;
            return S_OK;
        }
        return E_INVALIDARG;
    }

    STDMETHOD(SaveState)(short i_index)
    {
        if ((size_t)i_index >= m_states.size())
            m_states.resize(i_index+1);
        m_states[i_index] = m_states[0];
        m_currentState = m_states.begin();
        return S_OK;
    }

    STDMETHOD(RestoreState)(short i_index)
    {
        if ((size_t)i_index >= m_states.size())
            return S_FALSE;

        m_states[0] = m_states[i_index];
        return S_OK;
    }
    ///@}

protected:
    typedef std::vector< State > StateContainer;
    StateContainer m_states;
    typename StateContainer::iterator m_currentState;

};
