// comstr.cpp is part of CQG COM Operators SDK Sample

// helper tools for printing/scanning into/from wide strings.
// provided as an example only, you're free to use own implementations.

#include	"stdafx.h"
#include	"comstr.h"

/*--------------------------------------------------------------------------------
 * CreateReturnString - Copy i_stringToReturn into a new string allocated
 * with CoTaskMemAlloc and make i_retString point to it.
 *---------------------------------------------------------------------------------*/
HRESULT CreateReturnString( wchar_t ** i_retString, const wchar_t * i_stringToReturn)
{
	size_t nBytes = ( wcslen( i_stringToReturn) + 1) * sizeof( wchar_t) ;

	// Allocate an outgoing string.

	wchar_t * pBuf = static_cast<wchar_t*>( CoTaskMemAlloc( nBytes)) ;
	if( pBuf == NULL)
	{
		return E_OUTOFMEMORY ;
	}
	
	// Copy string to output buffer.
	wcscpy( pBuf, i_stringToReturn) ;

	*i_retString = pBuf ;

	return S_OK ;
}

/*---------------------------------------------------------------------------------
 * ConstantFloat - if the string can be interpreted as a floating-point number
 * with no significant trailing string, that value is returned.  Otherwise,
 * INVALID_VALUE is returned.
 *---------------------------------------------------------------------------------*/
double ConstantFloat( wchar_t * i_string)
{
	int i;
    for( i = 0; i_string[i]; ++i)
		if( !iswspace( i_string[i]))
			break;

	if( !i_string[i])
		return( INVALID_VALUE);					/* empty string */

	wchar_t * endPtr;

	double v = wcstod( i_string, &endPtr);

	for( i = 0; endPtr[i]; ++i)
		if( !iswspace( endPtr[i]))
			return( INVALID_VALUE);				/* significant trailing string */

	return( v);
}

/*---------------------------------------------------------------------------------
 * ConstantInt - if the string can be interpreted as an integer number
 * with no significant trailing string, that value is returned.  Otherwise,
 * INVALID_INTEGER is returned.
 *---------------------------------------------------------------------------------*/
long ConstantInt( wchar_t * i_string)
{
	int i;
    for( i = 0; i_string[i]; ++i)
		if( !iswspace( i_string[i]))
			break;

	if( !i_string[i])
		return( INVALID_INTEGER);					/* empty string */

	wchar_t * endPtr;

	long l = wcstol( i_string, &endPtr, 10);

	for( i = 0; endPtr[i]; ++i)
		if( !iswspace( endPtr[i]))
			return( INVALID_INTEGER);				/* significant trailing string */

	return( l);
}

/*---------------------------------------------------------------------------------
 * NegateExpr - the string i_dest is equal to "-(" + i_src + ")".  There must
 * be sufficient memory allocated to i_dest.
 *---------------------------------------------------------------------------------*/
void NegateExpr( wchar_t * i_dest, wchar_t * i_src)
{
	size_t srcLength = wcslen( i_src);

	i_dest[0] = L'-';
	i_dest[1] = L'(';
	wcsncpy( i_dest + 2, i_src, srcLength);
	i_dest[srcLength + 2] = L')';
	i_dest[srcLength + 3] = L'\0';
}

/*---------------------------------------------------------------------------------
 * RemoveNegate - if the string i_src is prefixed with L"-(" and suffixed with
 * L")", these substrings are removed in i_dest.  Otherwise, i_dest is a copy
 * of i_src.  There must be sufficient memory allocated to i_dest.
 *---------------------------------------------------------------------------------*/
void RemoveNegate( wchar_t * i_dest, wchar_t * i_src)
{
	size_t srcLength = wcslen( i_src);
	if( IsNegated( i_src))
		{
		wcsncpy( i_dest, i_src + 2, srcLength - 3);
		i_dest[srcLength - 3] = L'\0';
		}
	else
		{
		wcscpy( i_dest, i_src);
		}
}

/*--------------------------------------------------------------------------------
 * IsNegated - Returns TRUE if the expression is prefixed with L"-(" and suffixed
 * with L')', otherwise returns FALSE;
 *--------------------------------------------------------------------------------*/
BOOL IsNegated( wchar_t * i_src)
{
	size_t srcLength = wcslen( i_src);
	if( srcLength >= 3
		&& i_src[0] == L'-'
		&& i_src[1] == L'('
		&& i_src[srcLength - 1] == L')')
		return( TRUE);
	return( FALSE);
}
