// CurrentSecond.h is part of CQG COM Operators SDK Sample

#pragma once

#include "resource.h"       // main symbols

#include "CurrentSecondSample.h"
#include "MyOperatorBase.h"

struct CCurrentSecondState
{
   long secondInMinute;

   CCurrentSecondState()
      : secondInMinute(INVALID_INTEGER)
   {
   }

};


/// @class CCurrentSecond implements CQG COM operator with ability to initiate updates.
/// This operator returns current second of day and updates it with the period as parameter.
class ATL_NO_VTABLE CCurrentSecond
   : public CComObjectRootEx<CComSingleThreadModel>
   , public CComCoClass<CCurrentSecond, &CLSID_CurrentSecond>
   , public CMyOperatorBase<CCurrentSecondState>
   , public ICQGOp2
   , public ICQGOpUpdateSource
{
   typedef CMyOperatorBase<CCurrentSecondState> MyBase;

public:
   CCurrentSecond();
   ~CCurrentSecond();

   DECLARE_REGISTRY_RESOURCEID(IDR_CURRENTSECOND)

   DECLARE_NOT_AGGREGATABLE(CCurrentSecond)

   BEGIN_COM_MAP(CCurrentSecond)
       COM_INTERFACE_ENTRY(ICQGOp)
       COM_INTERFACE_ENTRY(ICQGOp2)
       COM_INTERFACE_ENTRY(ICQGOpUpdateSource)
   END_COM_MAP()

   DECLARE_PROTECT_FINAL_CONSTRUCT()

   HRESULT FinalConstruct()
      { return S_OK; }

   void FinalRelease()
      { }

public:

   ///@name ICQGOp interface methods
   ///@{
   STDMETHOD(GetTitle)(LPWSTR * o_title, LPWSTR * o_error);
   STDMETHOD(Initialize)(short i_ii, LPWSTR i_expression, LPWSTR * o_error);
   STDMETHOD(GetInputCount)(short * o_inputCount, LPWSTR * o_error);
   STDMETHOD(GetOutputCount)(short * o_outputCount, LPWSTR * o_error);
   STDMETHOD(GetOutputBarType)(short i_ii, LPWSTR * o_barTypeText, LPWSTR * o_error);
   STDMETHOD(GetParameterCount)(short * o_parmCount, LPWSTR * o_error);
   STDMETHOD(GetParameter)(short i_ii, LPWSTR * o_parameter, LPWSTR * o_error);
   STDMETHOD(GetExtraStepCount)(long * o_extraStepCount, LPWSTR * o_error);
   STDMETHOD(GetOutputScaleType)(short i_ii, short * o_scaleType, short * o_index1, short * o_index2, LPWSTR * o_error);
   ///@}

   ///@name ICQGOp2 interface methods
   ///@{
   
   STDMETHOD(InitializeStep(int IsCalcLatest, int i_updateStepIndex, int i_sizeIn, double * i_lastValidInput));
   STDMETHOD(Calculate(int i_stepIndex, int i_sizeIn, double * i_input, int i_sizeOut, double * o_output));
   STDMETHOD(InitializeConstants(int i_sizeIn, double * i_input));

   STDMETHOD(NeedsInputsFilledIn(int * o_returnFlag))
      { *o_returnFlag = TRUE; return S_OK; }
   STDMETHOD(CanHaveGapsInOutputs(int * o_returnFlag))
      { *o_returnFlag = TRUE; return S_OK; }
   STDMETHOD(IsDependentOnStartTime(int * o_returnFlag))
      { *o_returnFlag = TRUE; return S_OK; }
   STDMETHOD(ScrollValueSet(int * o_returnFlag))
      { *o_returnFlag = 3; return S_OK; }

   ///@}

   ///@name ICQGOpUpdateSource interface methods
   ///@{
   STDMETHOD(RegisterUpdateSink(ICQGOpUpdateSink *i_sink));
   STDMETHOD(UnregisterUpdateSink(ICQGOpUpdateSink *i_sink));
   ///@}

private:

   CComQIPtr<ICQGOpUpdateSink> m_updateSink;
   long m_updatePeriodSeconds;

   HANDLE m_timerQueue;
   HANDLE m_timer;


};

OBJECT_ENTRY_AUTO(__uuidof(CurrentSecond), CCurrentSecond)
