// MyMinMax.cpp is part of CQG COM Operators SDK Sample

#include "stdafx.h"
#include <limits>
#include <algorithm>

#include "MyMinMax.h"
#include "comstr.h"


// CMyMinMax
CMyMinMax::CMyMinMax()
    : m_period(INVALID_INTEGER)
{
}

STDMETHODIMP CMyMinMax::GetTitle(LPWSTR * o_title, LPWSTR * o_error)
{
    wchar_t s[128];
    _snwprintf(s, sizeof(s)/sizeof(s[0]), L"MyMinMax(%%s, %d)", m_period);
    return CreateReturnString(o_title, s);
}

STDMETHODIMP CMyMinMax::Initialize(short i_ii, LPWSTR i_expression, LPWSTR * o_error)
{
   if (i_ii == 1) 
   {
      m_period = ConstantInt(i_expression);
      if (m_period <= 0 || m_period > 10000)
      {
         m_period = INVALID_INTEGER;
      }

      if (m_period == INVALID_INTEGER)
      {
         m_period = 0;

         CreateReturnString(o_error, L"Bad period parameter - it should be in range [1, 10000]");
         return E_INVALIDARG;
      }
   }

   return S_OK;
}

STDMETHODIMP CMyMinMax::GetInputCount(short * o_inputCount, LPWSTR * o_error)
{
    if (o_inputCount)
    {
        *o_inputCount = 1;
        return S_OK;
    }

    return E_INVALIDARG;
}

STDMETHODIMP CMyMinMax::GetOutputCount(short * o_outputCount, LPWSTR * o_error)
{
    if (o_outputCount)
    {
        *o_outputCount = 2;
        return S_OK;
    }

    return E_INVALIDARG;
}

STDMETHODIMP CMyMinMax::GetOutputBarType(short i_ii, LPWSTR * o_barTypeText, LPWSTR * o_error)
{
    if (i_ii==0)
        return CreateReturnString( o_barTypeText, L"curve1");
    if (i_ii==1)
        return CreateReturnString( o_barTypeText, L"curve2");
    return S_FALSE;
}

STDMETHODIMP CMyMinMax::GetParameterCount(short * o_parmCount, LPWSTR * o_error)
{
    if (o_parmCount)
    {
        *o_parmCount = 1;
        return S_OK;
    }
    return E_INVALIDARG;
}

STDMETHODIMP CMyMinMax::GetParameter(short i_ii, LPWSTR * o_parameter, LPWSTR * o_error)
{
    wchar_t s[128] = L"";
    if (i_ii == 0) 
    {
        _snwprintf(s, sizeof(s)/sizeof(s[0]), L"%d", m_period);
        return CreateReturnString(o_parameter, s);
    }
    return E_INVALIDARG;
}

STDMETHODIMP CMyMinMax::GetExtraStepCount(long * o_extraStepCount, LPWSTR * o_error)
{
    if (o_extraStepCount)
    {
        *o_extraStepCount = m_period;
        return S_OK;
    }

    return E_INVALIDARG;
}

STDMETHODIMP CMyMinMax::InitializeStep(int i_sizeIn, double * i_lastValidInput)
{
    if (i_sizeIn < 1)
        return E_INVALIDARG;

    m_currentState->lastValidInput = (i_sizeIn >= 1)? i_lastValidInput[0] : INVALID_VALUE;
    return S_OK;
}

STDMETHODIMP CMyMinMax::Calculate(int i_sizeIn, double * i_input, int i_sizeOut, double * o_output)
{
    if (i_sizeIn < 1 || i_sizeOut < 2)
        return E_INVALIDARG;

    o_output[0] = o_output[1] = INVALID_VALUE;
    
    double lastInput = i_input[0];
    if (lastInput==INVALID_VALUE)
        lastInput = m_currentState->lastValidInput;
    if (lastInput==INVALID_VALUE)
        return S_OK;

    o_output[0] = (std::numeric_limits<double>::max)();
    o_output[1] = (std::numeric_limits<double>::min)();
    m_currentState->lastValidInput = lastInput;
    if (m_currentState->sequenceBuffer.size() >= (size_t)m_period)
        m_currentState->sequenceBuffer.erase(m_currentState->sequenceBuffer.begin());
    m_currentState->sequenceBuffer.push_back(lastInput);
    for(std::vector< double >::const_iterator cit = m_currentState->sequenceBuffer.begin(); cit != m_currentState->sequenceBuffer.end(); ++cit)
    {
        o_output[0] = ( std::min )(*cit, o_output[0]);
        o_output[1] = ( std::max )(*cit, o_output[1]);
    }
    return S_OK;
}

