// MyMinMax.h is part of CQG COM Operators SDK Sample

#pragma once
#include <vector>
#include "resource.h"       // main symbols

#include "MyMinMaxSample.h"
#include "MyOperatorBase.h"

struct CMyMinMaxRangeState
{
   // empty, just for CMyOperatorBase reuse.

};

///@class CMyMinMaxRange uses sample operator MyMinMax outputs and returns their difference. This example
///    demonstrates how to reference already registered COM operator (MyMinMax in this case) and use it
///    from another COM operator.
class ATL_NO_VTABLE CMyMinMaxRange :
   public CComObjectRootEx<CComMultiThreadModel>,
   public CComCoClass<CMyMinMaxRange, &CLSID_MyMinMaxRange>,
   public CMyOperatorBase<CMyMinMaxRangeState>
{
public:
   CMyMinMaxRange();

DECLARE_REGISTRY_RESOURCEID(IDR_MYMINMAXRANGE)

DECLARE_NOT_AGGREGATABLE(CMyMinMaxRange)

BEGIN_COM_MAP(CMyMinMaxRange)
    COM_INTERFACE_ENTRY(ICQGOp)
END_COM_MAP()

   DECLARE_PROTECT_FINAL_CONSTRUCT()

   HRESULT FinalConstruct()
      { return S_OK; }

   void FinalRelease()
      { }

public:
    ///@name ICQGOp interface methods
    ///@{
    STDMETHOD(GetTitle)(LPWSTR * o_title, LPWSTR * o_error);
    STDMETHOD(Initialize)(short i_ii, LPWSTR i_expression, LPWSTR * o_error);
    STDMETHOD(GetInputCount)(short * o_inputCount, LPWSTR * o_error);
    STDMETHOD(GetOutputCount)(short * o_outputCount, LPWSTR * o_error);
    STDMETHOD(GetOutputBarType)(short i_ii, LPWSTR * o_barTypeText, LPWSTR * o_error);
    STDMETHOD(GetParameterCount)(short * o_parmCount, LPWSTR * o_error);
    STDMETHOD(GetParameter)(short i_ii, LPWSTR * o_parameter, LPWSTR * o_error);
    STDMETHOD(GetExtraStepCount)(long * o_extraStepCount, LPWSTR * o_error);
    STDMETHOD(InitializeStep)(int i_sizeIn, double * i_lastValidInput);
    STDMETHOD(Calculate)(int i_sizeIn, double * i_input, int i_sizeOut, double * o_output);
    STDMETHOD(SaveState)(short i_index);
    STDMETHOD(RestoreState)(short i_index);
    ///@}

private:
   /// @brief this is private helper method for quick check the consistency of our nested operator pointer.
   bool isOK() const
      { return !!m_minmaxOp; }

   CComPtr<ICQGOp> m_minmaxOp; ///< ptr to nested COM operator (see sample MyMinMax) we reference for our calculations.

};

OBJECT_ENTRY_AUTO(__uuidof(MyMinMaxRange), CMyMinMaxRange)
