// MyRangeMarker.cpp is part of CQG COM Operators SDK Sample

#include "stdafx.h"
#include <limits>

#include "MyRangeMarker.h"
#include "comstr.h"


// CMyRangeMarker
CMyRangeMarker::CMyRangeMarker()
   : m_cqgicMaxExtraSteps(CQGIC_DEFAULT_MAX_EXTRA_STEPS)
   , m_frameMinutes(INVALID_INTEGER)
   , m_enableShading(true)
   , m_colorShading(RGB(255, 0, 0))
   , m_primitivesChanged(false)
   , m_enableLabel(false)
   , m_enableLabelNormalize(false)
   , m_tickSize(INVALID_VALUE)
{
}

STDMETHODIMP CMyRangeMarker::GetTitle(LPWSTR * o_title, LPWSTR * o_error)
{
   wchar_t s[128] = L"";

   // NOTE amount of %%s qualifiers shall correspond to GetInputCount call result
   _snwprintf(s, sizeof(s)/sizeof(s[0]), L"MyRangeMarker(%%s, %%s, %d, %d, %d, %d, %d)",
      m_frameMinutes, m_enableShading ? 1 : 0, m_colorShading,
      m_enableLabel ? 1 : 0, m_enableLabelNormalize ? 1 : 0);

   return CreateReturnString(o_title, s);
}

STDMETHODIMP CMyRangeMarker::Initialize(short i_ii, LPWSTR i_expression, LPWSTR * o_error)
{
   // i_ii = 0 - input value (variable)
   // i_ii = 1 - input time (variable) see also comments in GetPrimitives about possible minute index input

   switch (i_ii)
   {
      case -1: // i_ii = -1 - CQGIC asks us to initialize 'max extra steps' constant (see comments for m_cqgicMaxExtraSteps)
         m_cqgicMaxExtraSteps = ConstantInt(i_expression);
         break;

      case 2: // i_ii = 2 - CQGIC asks us to initialize time frame range parameter
         m_frameMinutes = ConstantInt(i_expression);
         if (m_frameMinutes <= 0 || m_frameMinutes >= 1440) // intraday only
         {
            m_frameMinutes = INVALID_INTEGER;
            return E_INVALIDARG;
         }
         break;

      case 3: // i_ii = 3 - CQGIC asks us to initialize m_enableShading parameter
         m_enableShading = 0 != ConstantInt(i_expression);
         break;

      case 4: // i_ii = 4 - CQGIC asks us to initialize m_colorShading parameter
         m_colorShading = ConstantInt(i_expression);
         break;

      case 5: // i_ii = 5 - CQGIC asks us to initialize m_enableLabel parameter
         m_enableLabel = 0 != ConstantInt(i_expression);
         break;

      case 6: // i_ii = 6 - CQGIC asks us to initialize m_enableLabelNormalize parameter
         m_enableLabelNormalize = 0 != ConstantInt(i_expression);
         break;

      default: // not required to handle other initializers
         break;

   }

   return S_OK;
}

STDMETHODIMP CMyRangeMarker::GetInputCount(short * o_inputCount, LPWSTR * o_error)
{
   if (o_inputCount)
   {
      *o_inputCount = 2; // input value, input time
      return S_OK;
   }

   return E_INVALIDARG;
}

STDMETHODIMP CMyRangeMarker::GetOutputCount(short * o_outputCount, LPWSTR * o_error)
{
   if (o_outputCount)
   {
      // we output only 1 value (our operator just passes through the first input value),
      // and primitive output is never counted as specific output.
      *o_outputCount = 1; 
      return S_OK;
   }

   return E_INVALIDARG;
}

STDMETHODIMP CMyRangeMarker::GetOutputBarType(short i_ii, LPWSTR * o_barTypeText, LPWSTR * o_error)
{
   if (i_ii == 0)
   {
      return CreateReturnString(o_barTypeText, L"curve1");
   }

   return S_FALSE;
}

STDMETHODIMP CMyRangeMarker::GetParameterCount(short * o_parmCount, LPWSTR * o_error)
{
   if (o_parmCount)
   {
      // GetParameterCount is supposed to return amount of constant operands.
      // we have 5: m_frameMinutes, m_enableShading, m_colorShading, m_enableLabel, m_enableLabelNormalize
      *o_parmCount = 5;
      return S_OK;
   }

   return E_INVALIDARG;
}

STDMETHODIMP CMyRangeMarker::GetParameter(short i_ii, LPWSTR * o_parameter, LPWSTR * o_error)
{
   wchar_t s[128] = L"";

   // the i_ii passed in here is relative to zero. we shall get here i_ii from range [0; GetParameterCount() - 1]
   switch (i_ii)
   {
      case 0: // i_ii = 0 - CQGIC asks us to reply value of m_frameMinutes
         _snwprintf(s, sizeof(s) / sizeof(s[0]), L"%d", m_frameMinutes);
         break;

      case 1: // i_ii = 1 - CQGIC asks us to reply value of m_enableShading
         _snwprintf(s, sizeof(s) / sizeof(s[0]), L"%d", m_enableShading ? 1 : 0);
         break;

      case 2: // i_ii = 2 - CQGIC asks us to reply value of m_colorShading
         _snwprintf(s, sizeof(s) / sizeof(s[0]), L"%d", m_colorShading);
         break;

      case 3: // i_ii = 3 - CQGIC asks us to reply value of m_enableLabel
         _snwprintf(s, sizeof(s) / sizeof(s[0]), L"%d", m_enableLabel ? 1 : 0);
         break;

      case 4: // i_ii = 4 - CQGIC asks us to reply value of m_enableLabelNormalize
         _snwprintf(s, sizeof(s) / sizeof(s[0]), L"%d", m_enableLabelNormalize ? 1 : 0);
         break;

      default: // CQGIC asks about something we don't know
         return E_INVALIDARG;
         break;

   }

   return CreateReturnString(o_parameter, s);
}

STDMETHODIMP CMyRangeMarker::GetExtraStepCount(long * o_extraStepCount, LPWSTR * o_error)
{
   if (o_extraStepCount)
   {
      *o_extraStepCount = m_cqgicMaxExtraSteps; // see comments in header
      return S_OK;
   }

   return E_INVALIDARG;
}

STDMETHODIMP CMyRangeMarker::InitializeStep(int IsCalcLatest, int i_updateStepIndex, int i_sizeIn, double * i_lastValidInput)
{
   UNREFERENCED_PARAMETER(IsCalcLatest);
   UNREFERENCED_PARAMETER(i_updateStepIndex);

   if (i_sizeIn < 2)
   {
      return E_INVALIDARG;
   }

   m_primitivesChanged = false;

   m_currentState->ranges.clear();
   m_currentState->timeAndValueBuffer.clear();

   m_currentState->lastValidInput.first = i_lastValidInput[0];
   m_currentState->lastValidInput.second = (int)i_lastValidInput[1];

   return S_OK;
}

STDMETHODIMP CMyRangeMarker::Calculate(int i_stepIndex, int i_sizeIn, double * i_input, int i_sizeOut, double * o_output)
{
   if (i_sizeIn < 2 || i_sizeOut < 1 || m_frameMinutes <= 0)
   {
      return E_INVALIDARG;
   }

   o_output[0] = i_input[0]; // just pass through the input

   m_primitivesChanged = m_currentState->ProcessInput(
      i_input[0], (int)i_input[1], m_frameMinutes, m_cqgicMaxExtraSteps)
         || m_primitivesChanged;

   return S_OK;
}

STDMETHODIMP CMyRangeMarker::InitializeConstants(int i_sizeIn, double * i_input)
{
   // CQGIC gives a chance to initialize following constants here:
   // i_input[0] - minimum value change (e.g. tick size for price scales)
   // i_input[1] - price to dollar multiplier (for price scales)

   // let's use first one item to print integer labels
   m_tickSize = i_sizeIn > 0 ? i_input[0] : INVALID_VALUE;

   return S_OK;
}

STDMETHODIMP CMyRangeMarker::HavePrimitivesChanged(short * o_changed)
{
   // CQGIC calls HavePrimitivesChanged to understand whether or not primitives
   // were changed during last stepping calculation cycle.

   if (o_changed)
   {
      *o_changed = m_primitivesChanged ? 1 : 0;
      return S_OK;
   }

   return E_INVALIDARG;
}

STDMETHODIMP CMyRangeMarker::GetPrimitives(ChartTime i_startTime, ChartTime i_endTime,
   int * o_count, Primitive * * o_primitives)
{
   // CQGIC calls GetPrimitives to ask operator about primitives in the interval [i_startTime; i_endTime). i_startTime and i_endTime 
   // contain index within minute and millisecond members, but it matters only for non-timed bars. see comments below. o_primitives
   // points to the COM memory to be allocated by this method and to be filled with flat array of Primitive structures.
   // o_count points to amount of Primitive structures placed in *o_primitives

   if (!o_count || !o_primitives)
   {
      return E_INVALIDARG;
   }

   // NOTE indexes within minute are important for non-timed bars like CVB/TFlow etc. - we may have any amount
   // of bars in one minute there. our sample operator ignores indexes and marks all with the range of 1-minute resolution.
   // if one would like to have indexes taken in account, he'd use additional variable input of following form: "TimeOf(@): index"

   if (m_states.end() == m_currentState)
   {
      ATLASSERT(FALSE);
      return S_FALSE;
   }

   std::vector<Primitive> primitives;
   for (size_t i = 0; i < m_currentState->ranges.size(); ++i)
   {
      if (m_currentState->ranges[i].HasIntersection(i_startTime.minuteSince1900, i_endTime.minuteSince1900))
      {
         m_currentState->ranges[i].AppendPrimitives(primitives,
            m_enableShading, m_colorShading, m_enableLabel,
            m_enableLabelNormalize, m_tickSize);
      }
   }

   if (primitives.empty())
   {
      *o_primitives = NULL;
      *o_count = 0;

      return S_OK;
   }

   *o_count = (int)primitives.size();

   CComHeapPtr<Primitive> p;
   p.Allocate(primitives.size());

   *o_primitives = p.Detach();

   if (*o_primitives)
   {
      memcpy(*o_primitives, &primitives[0], sizeof(primitives[0]) * primitives.size());
   }
   else
   {
      *o_count = 0;
   }

   return S_OK;
}

bool Range::AppendPrimitives(std::vector<Primitive>& io_primitives,
   bool i_shading, COLORREF i_colorShading, bool i_label, bool i_norm, double i_tick) const
{
   bool res = false;

   if (i_shading)
   {
      Primitive prim;
      ZeroMemory(&prim, sizeof(Primitive));

      prim.type = ePrimitiveRectangle;

      prim.pen.style = PS_ALTERNATE;
      prim.pen.width = 1;
      prim.pen.color = i_colorShading;

      prim.brush.style = BS_NULL;
      prim.brush.hatch = HS_HORIZONTAL;
      prim.brush.color = i_colorShading;

      prim.pt1.time.minuteSince1900 = timeStart;
      prim.pt1.value = hi;

      prim.pt2.time.minuteSince1900 = timeEndInclusive;
      prim.pt2.value = lo;

      prim.symbolSizeFactor = 1;
      prim.labelSizeFactorMultiplier = 1.0;

      if (i_label)
      {
         OLECHAR label[100] = { 0 };

         if (INVALID_VALUE != i_tick && i_tick != 0.0)
         {
            int height = int((hi - lo) / i_tick);

            swprintf_s(label, sizeof(label) / sizeof(label[0]), L"%d", height);
            prim.label = ::SysAllocString(label);

            if (i_norm)
            {
               // NOTE simplest way used, just as example. better to calculate something more reasonable in order
               // to not go out of adequate font size ranges. this would require analysis of min/max deltas
               // and use of sizeFactor field.
               prim.labelSizeFactorMultiplier = height > 100. ? 10. : height / 10.;
            }
         }
         else
         {
            // if don't have tick size, use raw value and no label normalizing
            swprintf_s(label, sizeof(label) / sizeof(label[0]), L"%.2f", hi - lo);
            prim.label = ::SysAllocString(label);
         }

         // let's align label by the center of the rectangle marquee
         prim.horzLabelAlign = eHAlignmentCenter;
         prim.vertLabelAlign = eVAlignmentCenter;
      }

      res = true;
      io_primitives.push_back(prim);
   }

   return res;
}

bool CMyRangeMarkerState::ProcessInput(double i_newValue, int i_newTimeMinutes, int i_frameMinutes, size_t i_maxBufSize)
{
    if (i_newValue == INVALID_VALUE)
    {
       i_newValue = lastValidInput.first;
    }

    if (i_newTimeMinutes == INVALID_INTEGER)
    {
       i_newTimeMinutes = lastValidInput.second;
    }

    if (i_newValue == INVALID_VALUE || i_newTimeMinutes == INVALID_INTEGER)
    {
       // unsuccessfull fill-in of invalids - say we aren't changed at all.
       return false;
    }

    lastValidInput.first = i_newValue;
    lastValidInput.second = i_newTimeMinutes;

    timeAndValueBuffer.push_back(lastValidInput);
    if (timeAndValueBuffer.size() > i_maxBufSize)
    {
       timeAndValueBuffer.pop_front();
    }

    // NOTE code below supposes that second input not decreases (as time grows)

    bool res = false;
    if (ranges.empty()) // no ranges, look if can start first one
    {
       if (timeAndValueBuffer.size() > 1)
       {
          int prevTimeMinutes = timeAndValueBuffer[timeAndValueBuffer.size() - 2].second;
          if (prevTimeMinutes / i_frameMinutes < i_newTimeMinutes / i_frameMinutes)
          {
             // we switched frame during last bar - able to stably start first range
             ranges.push_back(Range(i_newValue, i_newValue, i_newTimeMinutes, i_newTimeMinutes + i_frameMinutes));
             res = true;
          }
       }
    }
    else if (i_newTimeMinutes >= ranges.back().timeEndExclusive) // start next range
    {
       // create new range
       ranges.push_back(Range(i_newValue, i_newValue, i_newTimeMinutes, i_newTimeMinutes + i_frameMinutes));
       res = true;
    }
    else // update last range if any
    {
       res = ranges.back().Update(i_newValue, i_newTimeMinutes);
    }

    return res;
}
